<?php
/*
Plugin Name: WP-OpenID-UPV
Version: 1.0.0
Plugin URI: https://blogs.upv.es
Description: Plugin to integrate WordPress authentication with OpenID
Author: Jorge Ferrando
*/

$error_reporting = error_reporting(0); // hide any warnings when attempting to fetch the optional config file
include_once( dirname(__FILE__).'/wpopenid-conf.php' ); // attempt to fetch the optional config file
error_reporting( $error_reporting ); // unhide warnings
//ini_set('display_errors', 'On');
//error_reporting(E_ALL);

// Validate configuration:
$openid_configured = true;
if( !is_array( $wpopenid_options )){
	$openid_configured = false;
}

else if ($wpopenid_options['include_path'] == ''){
	$openid_configured = false;
}

else if ((include_once $wpopenid_options['include_path']) != true)
   $openid_configured = false;

else if (!isset($wpopenid_options['identity']))
	$openid_configured = false;

// plugin hooks into authentication system
add_action('init', 'wpopenid_init');
add_action('wp_authenticate', 'wpopenid_authenticate', 10, 2);
add_action('wp_logout', 'wpopenid_logout');
add_action('lost_password', 'disable_function');
add_action('retrieve_password', 'disable_function');
add_action('check_passwords', 'wpopenid_check_passwords', 10, 3);
add_action('password_reset', 'wpopenid_disable_function');
add_filter('show_password_fields', 'wpopenid_show_password_fields');


$openid = NULL;

function wpopenid_init() {
   global $openid, $openid_configured, $wpopenid_options;
   #if ( !$openid_configured )
   #   die( __( 'wpOpenID plugin not configured', 'wpopenid' ));
   
   $openid = new LightOpenID($_SERVER["SERVER_NAME"]);
   $openid->verify_peer= true;
   $openid->cainfo= $wpopenid_options['library_path'] . 'ca-bundle.pem';   // certificados raíz aceptados
   $openid->capath= $wpopenid_options['library_path'];                      // desactivar certificados raíz del sistema
   $openid->cnmatch= 'www.rediris.es';
}

function wpopenid_authenticate($gatewayMode=false) {
   global $openid, $wpopenid_options, $openid_configured;

   if ( !$openid_configured )
      die( __( 'wpOpenID plugin not configured', 'wpopenid' ));

   try {
       // comprobamos la fase en la que entramos en esta página
       if(!$openid->mode) {
           // comenzamos el proceso de autenticación
           $openid->identity= $wpopenid_options['identity'];
           $openid->optional = array('contact/email');
           header('Location: ' . $openid->authUrl());
       } elseif($openid->mode == 'cancel') {
           // el usuario ha cancelado el proceso
           // acciones cuando se cancela el proceso
       } else {
           // se ha terminado la autenticación
           if ($openid->validate()) {
               // validación correcta
               $idOpenID= $openid->identity;
               if (preg_match('#^https://yo\.rediris\.es/soy/([-_\w]+)@upv\.es/?$#', $idOpenID, $usrOpenID)) {
                  $atOpenID= $openid->getAttributes();
                  if (!empty ($atOpenID["contact/email"]))
                       $email= $atOpenID["contact/email"];
                  $login = $usrOpenID[1];

                  if ($user = get_userdatabylogin($login)){ // user already exists
                     // the CAS user has a WP account
                     wp_set_auth_cookie( $user->ID );

                     // Allow custom user profile sync
                     if (function_exists( 'wpopenid_syncuser' ))
                        wpopenid_syncuser($login, $email);

                     if( isset( $_GET['redirect_to'] )){
                        wp_redirect( preg_match( '/^http/', $_GET['redirect_to'] ) ? $_GET['redirect_to'] : site_url( $_GET['redirect_to'] ));
                        die();
                     }
                     wp_redirect( site_url( '/' ));
                     die();

                  }else{
                     // the CAS user _does_not_have_ a WP account
                     if (function_exists( 'wpopenid_nowpuser' ))
                        wpopenid_nowpuser($login);
                     else
                        die( __( 'you do not have permission here', 'wpopenid' ));
                  }
                  header('debug', $login);
               }
               else {
                   // validación correcta, pero el usuario no es de la UPV
                   // acciones cuando no es usuario UPV
               }
           }
           else {
               // validación no correcta
               // acciones cuando falla la autenticación
           }
       }
   } 
   
   catch(ErrorException $e) {
       echo $e->getMessage();
   }

}


// hook CAS logout to WP logout
function wpopenid_logout() {
   global $openid_configured;

   if (!$openid_configured)
      die( __( 'wpOpenID plugin not configured', 'wpcas' ));

   wp_redirect(home_url());
   exit();
}

// hide password fields on user profile page.
function wpopenid_show_password_fields( $show_password_fields ) {
   return false;
}

// disabled reset, lost, and retrieve password features
function wpopenid_disable_function() {
   die( __( 'Sorry, this feature is disabled.', 'wpcas' ));
}

// set the passwords on user creation
// patched Mar 25 2010 by Jonathan Rogers jonathan via findyourfans.com
function wpopenid_check_passwords( $user, $pass1, $pass2 ) {
   $random_password = substr( md5( uniqid( microtime( ))), 0, 8 );
   $pass1=$pass2=$random_password;
}

